#!/bin/sh
# Run this at the beginning of any script using PhdigetSBC config
# This provides functions to do common things

# config file locations, version
SBC_USERAPPS=/usr/userapps; export SBC_USERAPPS
SBC_CONFIGDIR=/etc/webif; export SBC_CONFIGDIR

DEBIAN_VERSION=`cat /etc/debian_version`
DEBIAN_MAJOR_VERSION=${DEBIAN_VERSION%%.*}

# creates interfaces file from info in network.conf
apply_network_config()
{
	# logic to make sure we have a valid configuration
	mkdir -p /etc/network

	. $SBC_CONFIGDIR/network.conf
	if [ $lanconfig = "static" ]; then
		if [ -z $ipaddr ]; then
			return 1
		fi
		if [ -z $subnetmask ]; then
			return 1
		fi
		if [ -z $gateway ]; then
			return 1
		fi
		# dns has to be manual if using static IP
		if [ $dnsconfig != "manual" ]; then
			return 1
		fi
	else
		# only other option is dhcp
		if [ $lanconfig != "dhcp" ]; then
			return 1
		fi
	fi
	
	. $SBC_CONFIGDIR/wireless_network.conf
	if [ $lanconfig = "static" ]; then
		if [ -z $ipaddr ]; then
			return 1
		fi
		if [ -z $subnetmask ]; then
			return 1
		fi
		if [ -z $gateway ]; then
			return 1
		fi
		# dns has to be manual if using static IP
		if [ $dnsconfig != "manual" ]; then
			return 1
		fi
	else
		# only other option is dhcp
		if [ $lanconfig != "dhcp" ]; then
			return 1
		fi
	fi

	# write out a new interfaces file

	cat > /etc/network/interfaces.new << EOF
# interfaces(5) file used by ifup(8) and ifdown(8)
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN

# Configure Loopback
auto lo
iface lo inet loopback
		
EOF

	. $SBC_CONFIGDIR/wireless_network.conf
	if [ $lanconfig = "static" ]; then
		cat >> /etc/network/interfaces.new << EOF
# Default wireless config, static ip
iface default inet static
	address $ipaddr
	netmask $subnetmask
	gateway $gateway
EOF
	else
		cat >> /etc/network/interfaces.new << EOF
# Default wireless config, dhcp
iface default inet dhcp
EOF
	fi
	cat >> /etc/network/interfaces.new << EOF
	metric 10
EOF
	
	if [ "$dnsconfig" = "manual" ]; then
		cat >> /etc/network/interfaces.new << EOF
	dns-nameservers $dnsserver1 $dnsserver2
EOF
	fi
	
	echo "" >> /etc/network/interfaces.new

	. $SBC_CONFIGDIR/network.conf
	if [ $lanconfig = "static" ]; then
		cat >> /etc/network/interfaces.new << EOF
# Wired brought up/down by ifplugd, static ip
iface eth0 inet static
	address $ipaddr
	netmask $subnetmask
	gateway $gateway
EOF
	else
		cat >> /etc/network/interfaces.new << EOF
# Wired brought up/down by ifplugd, dhcp
iface eth0 inet dhcp
EOF
	fi
	
	if [ "$dnsconfig" = "manual" ]; then
		cat >> /etc/network/interfaces.new << EOF
	dns-nameservers $dnsserver1 $dnsserver2
EOF
	fi

	# overwrite old interfaces file
	mv /etc/network/interfaces.new /etc/network/interfaces
	
	# now, ssh
	if [ "$sshstart" = "true" ]; then
		[ ! -e /etc/ssh/ssh_host_dsa_key ] && [ ! -e /etc/ssh/ssh_host_ecdsa_key ] || [ ! -e /etc/ssh/ssh_host_rsa_key ] && {
			dpkg-reconfigure -f noninteractive openssh-server
		}
		if [ ${DEBIAN_MAJOR_VERSION} -ge 9 ] ; then
			systemctl is-enabled ssh 2>&1 1>/dev/null || {
				systemctl enable ssh;
				systemctl start ssh;
			}
		else
			[ "$(ls /etc/rc2.d/ | grep ssh)" = "" ] && update-rc.d ssh defaults
		fi
	else
		if [ ${DEBIAN_MAJOR_VERSION} -ge 9 ] ; then
			systemctl is-enabled ssh 2>&1 1>/dev/null && {
				systemctl stop ssh;
				systemctl disable ssh;
			}
		else
			[ "$(ls /etc/rc2.d/ | grep ssh)" != "" ] && update-rc.d ssh remove
		fi
	fi

	return 0
}

apply_packages_config()
{
	. /$SBC_CONFIGDIR/packages.conf
	if [ "$incldebian" = "true" ]; then
		cat /etc/apt/sources.list.d/multistrap-debian.list | sed 's/^#\(deb .*\)/\1/' > /etc/apt/sources.list.d/multistrap-debian.list.new
	else
		cat /etc/apt/sources.list.d/multistrap-debian.list | sed 's/^\(deb .*\)/#\1/' > /etc/apt/sources.list.d/multistrap-debian.list.new
	fi
		
	[ -e /etc/apt/sources.list.d/multistrap-debian.list.new ] && mv /etc/apt/sources.list.d/multistrap-debian.list.new /etc/apt/sources.list.d/multistrap-debian.list
}

apply_webservice_config()
{
	[ -e /etc/default/phidgetwebservice ] && [ -f $SBC_CONFIGDIR/phidgetwebservice.conf ] && cp $SBC_CONFIGDIR/phidgetwebservice.conf /etc/default/phidgetwebservice
}

apply_system_config()
{
	. /$SBC_CONFIGDIR/system.conf
	if [ "$zoneinfo" != "$( cat /etc/timezone )" ] || [ "$1" = "force" ]; then
		echo $zoneinfo > /etc/timezone
		[ -e /usr/share/zoneinfo/$zoneinfo ] && {
			echo $zoneinfo > /etc/timezone
			ln -fs /usr/share/zoneinfo/$zoneinfo /etc/localtime
		} || {
			echo "Etc/UTC" > /etc/timezone
			ln -fs /usr/share/zoneinfo/Etc/UTC /etc/localtime
		}
		#dpkg-reconfigure -f noninteractive tzdata
	fi
	
	if [ "$hostname" != "$( hostname )" ] || [ "$1" = "force" ]; then
		echo $hostname > /etc/hostname
		/bin/hostname -F /etc/hostname
		setup_avahi_services
		/etc/init.d/avahi-daemon reload
		[ -f /etc/init.d/phidgetwebservice ] && /etc/init.d/phidgetwebservice reload
		[ -f /etc/init.d/phidget22networkserver ] && /etc/init.d/phidget22networkserver reload
	fi
}

apply_webcam_config()
{
	[ -e /etc/mjpg_streamer ] && [ -f $SBC_CONFIGDIR/webcam.conf ] && cp $SBC_CONFIGDIR/webcam.conf /etc/mjpg_streamer/mjpg_streamer.conf
}

#creates Avahi services files
setup_avahi_services()
{
	_version=$( grep "version" /etc/phidgetsbc_version | awk '{print $2}' )
	_mac="$(cat /sys/class/net/eth0/address)"
	_board_name="$([ -e /proc/device-tree/model ] && (cat /proc/device-tree/model; echo '';) || (cat /proc/cpuinfo | awk '/^Hardware/ {print $3}'))"
	if [ "$_board_name" = "PhidgetSBC4" ]; then
		if [ -e /sys/class/i2c-adapter/i2c-2/2-0050/eeprom ] ; then
			_board_version="$(xxd -s 6 -l 2 -p /sys/class/i2c-adapter/i2c-2/2-0050/eeprom | awk '{print strtonum("0x"$1)}')"
		else
			_board_version="$(xxd -s 6 -l 2 -p /sys/class/i2c-adapter/i2c-1/1-0050/eeprom | awk '{print strtonum("0x"$1)}')"
		fi
	else
		_board_version="$(cat /proc/cpuinfo | awk '/^Revision/ {print strtonum("0x"$3)}')"
	fi
	_hostname=$(cat /etc/hostname)
	
	[ -e /etc/avahi/services/http.service.template ] && {
		eval "echo \"$(cat /etc/avahi/services/http.service.template)\"" > /etc/avahi/services/http.service.new
		mv /etc/avahi/services/http.service.new /etc/avahi/services/http.service
	}
	[ -e /etc/avahi/services/phidgetsbc.service.template ] && {
		eval "echo \"$(cat /etc/avahi/services/phidgetsbc.service.template)\"" > /etc/avahi/services/phidgetsbc.service.new
		mv /etc/avahi/services/phidgetsbc.service.new /etc/avahi/services/phidgetsbc.service
	}
}

# creates packages.conf
create_packages_config()
{
	cat > $SBC_CONFIGDIR/packages.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
incldebian="true"
EOF
}

# creates network.conf
create_network_config()
{
	cat > $SBC_CONFIGDIR/network.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
sshstart="false"
lanconfig="dhcp"
ipaddr=""
subnetmask=""
gateway=""
dnsconfig="auto"
dnsserver1=""
dnsserver2=""
EOF
}

# creates wireless_network.conf
create_wireless_network_config()
{
	cat > $SBC_CONFIGDIR/wireless_network.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
lanconfig="dhcp"
ipaddr=""
subnetmask=""
gateway=""
dnsconfig="auto"
dnsserver1=""
dnsserver2=""
EOF
}

# creates phidgetwebservice.conf
create_webservice_config()
{
	cat > $SBC_CONFIGDIR/phidgetwebservice.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
pws_enabled="false"
pws_port="5001"
pws_serverid=""
pws_password=""
EOF
}

# creates system.conf
create_system_config()
{
	cat > $SBC_CONFIGDIR/system.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
hostname="phidgetsbc"
zoneinfo="Etc/UTC"
EOF
}

# creates webcam.conf
create_webcam_config()
{
	cat > $SBC_CONFIGDIR/webcam.conf << EOF
#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN
webcam_framerate="25"
webcam_port="81"
webcam_wwwdir="/var/www/webcam"
webcam_res="320x240"
webcam_udevcontrol="false"
webcam_pass=""
EOF
}
