#!/bin/sh
#
# Default handlers for config files
#

. /lib/webif/config.sh

HANDLERS_config='
	webcam) reload_webcam;;
	system) reload_system;;
	network) reload_network;;
	packages) reload_packages;;
	phidgetwebservice) reload_webservice;;
	phidget22networkserver) reload_phidget22networkserver;;
	wireless_network) reload_wireless_network;;
'

mkdir -p "/tmp/.webif"
_pushed_dir=$(pwd)
cd "/tmp/.webif"

# edited-files/*		user edited files - stored with directory tree in-tact
#for edited_file in $(find "/tmp/.webif/edited-files/" -type f 2>&-); do
find "/tmp/.webif/edited-files/" -type f 2>&- | while read -r edited_file
do
	target_file=$(echo "$edited_file" | sed s/'\/tmp\/.webif\/edited-files'//g)
	fix_symlink_hack "$target_file"
	if tr -d '\r' <"$edited_file" >"$target_file"; then
		rm "$edited_file" 2>&-
	else
		ERROR="$ERROR Critical Error : Could not replace $target_file. Media full?<br />"
	fi
done
# leave if some files not applied
rm -r "/tmp/.webif/edited-files" 2>&-

# merges the tmp into the real
update_config() {
	# this reads in lines, so we can have spaces in config values
	while read -r config
	do
		key="${config%%=*}"
		val="${config#*=}"
		#echo "Key = $key, Val = $val"
		grep "^$key=" $SBC_CONFIGDIR/$1.conf && {
			grep -v "^$key=" $SBC_CONFIGDIR/$1.conf > $SBC_CONFIGDIR/$1.conf.new
			mv $SBC_CONFIGDIR/$1.conf.new $SBC_CONFIGDIR/$1.conf
		}
		echo "$key=$val" >> $SBC_CONFIGDIR/$1.conf
	done < /tmp/.config/$1.conf
	load_settings $1
	#IFS=$OLDIFS
}

reload_wireless_network() {
	wireless_interfaces=$( ifconfig -a|grep -E "wlan" |cut -d" " -f1 )
	
	for iface in "$wireless_interfaces"; do
		ifdown $iface
	done
	
	apply_network_config || ERROR="$ERROR WARNING: problem creating interfaces file<br />"
	
	for iface in "$wireless_interfaces"; do
		ifup $iface=default
	done
}

reload_network() {
	# stop ssh
	if [ "$sshstart" != "true" ]; then
		service ssh stop
	fi
	
	ifdown eth0
	apply_network_config || ERROR="$ERROR WARNING: problem creating interfaces file<br />"
	ifup eth0
	
	# start ssh
	if [ "$sshstart" = "true" ]; then
		service ssh start
	fi
}

reload_webcam() {
	/lib/udev/mjpg_streamer.agent stop /dev/video0
	apply_webcam_config
	/lib/udev/mjpg_streamer.agent udevstart /dev/video0
}

reload_system() {
	apply_system_config
}

reload_packages() {
	apply_packages_config
}

reload_webservice() {
	apply_webservice_config
	/etc/init.d/phidgetwebservice restart
}

reload_phidget22networkserver() {
	/etc/init.d/phidget22networkserver restart
}

mkdir -p "/tmp/.config"
cd /tmp/.config

# *.conf files

for config in $(ls *.conf 2>&-); do
	name=${config%.conf}
	update_config $name
	eval 'case "$name" in
		'"$HANDLERS_config"'
	esac'
done

# sync the system
sync

sleep 1
rm -f *.conf
