#!/bin/sh
#
# Webif^2 utility functions - X-Wrt
#
# This file is compatible with White Russian and Kamikaze.
#

#
# Misc. functions
#

# workarounds for stupid busybox slowness on [ ]
empty() {
	case "$1" in
		"") return 0 ;;
		*) return 255 ;;
	esac
}

equal() {
	case "$1" in
		"$2") return 0 ;;
		*) return 255 ;;
	esac
}

neq() {
	case "$1" in
		"$2") return 255 ;;
		*) return 0 ;;
	esac
}

# very crazy, but also very fast :-)
exists() {
	( < $1 ) 2>&-
}

fix_symlink_hack() {
	! touch "$1" >&- 2>&- && {
		local atmpfile
		atmpfile=$(mktemp "/tmp/webif-XXXXXX")
		cp -p "$1" "$atmpfile"
		equal "$?" "0" && {
			rm "$1"
			mv "$atmpfile" "$1"
		}
	}
}

remove_lines_from_file() {
	# $1=filename
	# $2=substring match indicating lines to remove (case sensitive)
	cat "$1" 2>/dev/null | grep -q "$2" && {
		fix_symlink_hack "$1"
		local _substr_sed
		_substr_sed=$(echo "$2" |  sed s/'\/'/'\\\/'/g)
		cat "$1" |  sed /"$_substr_sed"/d > "$1"
	}
}

# mktemp replacement that doesn't actually create the file (as busybox 1.3.1+ does)
mkuniqfilename() {
	local _lfile
	_lfile=$(mktemp $*)
	rm -f "$_lfile" 2>&- >&-
}

#
# Original config functions
#
#  These work with as a tuple based configuration system. apply.sh applies
#  the changes.
#
load_settings() {
	. /lib/webif/config.sh
	if [ $# -eq 1 ]; then
		exists $SBC_CONFIGDIR/$1.conf && . $SBC_CONFIGDIR/$1.conf
		exists /tmp/.config/$1.conf && . /tmp/.config/$1.conf
	fi
}
save_setting() {
	exists /tmp/.config/* || mkdir -p /tmp/.config
	oldval="$(eval "echo \${$2}")"
	grep "^$2=" /tmp/.config/$1.conf >&- 2>&- && {
		grep -v "^$2=" /tmp/.config/$1.conf > /tmp/.config/$1.conf.new 2>&- 
		mv /tmp/.config/$1.conf.new /tmp/.config/$1.conf 2>&- >&-
		oldval=""
	}
	equal "$oldval" "$3" || echo "$2=\"$3\"" >> /tmp/.config/$1.conf
}

# pconf
#  args: configfile, path, default
pconf_get() {
	if /usr/bin/phidgetconf -c $1 -g $2 1>/dev/null 2>&1; then
		/usr/bin/phidgetconf -c $1 -g $2;
	else
		echo $3;
	fi
}
#  args: configfile, path, type, value
pconf_set() {
	conf=$1
	path=$2
	type=$3
	shift
	shift
	shift
	val="$@";
	if [ "$type" = "B" ]; then
		[ "$val" = "true" ] && val="1"
		[ "$val" = "false" ] && val="0"
	fi
	[ "$type" = "S" ] && val="'$val'"
	cmd="/usr/bin/phidgetconf -c $conf -s $path -$type$val"
	eval "$cmd"
}
#  args: configfile, path
pconf_delete() {
	/usr/bin/phidgetconf -c $1 -d $2 1>/dev/null 2>&1
}

#
# validate form variables using validate.awk
#
validate() {
	if empty "$1"; then
		eval "$(awk -f /usr/lib/webif/validate.awk)"
	else
		eval "$(echo "$1" | awk -f /usr/lib/webif/validate.awk)"
	fi
}

set_mtd() 
{
	mtd=$( cat /proc/mtd | grep "\"$1\"" | awk {'print $1'} )
	mtd=${mtd%:}
	mtd="/dev/$mtd"
}

set_mtdblock() 
{
	mtd=$( cat /proc/mtd | grep "\"$1\"" | awk {'print $1'} )
	mtd=${mtd%:}
	mtd=${mtd#mtd}
	mtd="/dev/mtdblock$mtd"
}
