PATH=/usr/bin:/bin:/usr/sbin:/sbin

libdir=/usr/lib/webif
wwwdir=/var/www
cgidir=/var/www/cgi-bin
rootdir=/cgi-bin
indexpage=status-system.sh

. $libdir/functions.sh

#######################################################
# categories
#######################################################
categories() {
	grep '##WEBIF:' $cgidir/.categories $cgidir/*.sh 2>/dev/null | \
		awk -v "selected=$1" \
			-v "rootdir=$rootdir" \
			-v "indexpage=$indexpage" \
			-f $libdir/categories.awk -
}

#######################################################
# subcategories
#######################################################
subcategories() {
	grep -H "##WEBIF:name:$1:" $cgidir/*.sh 2>/dev/null | \
		sed -e 's,^.*/\([a-zA-Z0-9_\.\-]*\):\(.*\)$,\2:\1,' | \
		sort -n | \
		awk -v "selected=$2" \
			-v "rootdir=$rootdir" \
			-f $libdir/subcategories.awk -
}

#######################################################
# update_changes
#######################################################
update_changes() {
	CHANGES=$(($( (cat /tmp/.config/*.conf) 2>&- | wc -l)))
	EDITED_FILES=$(find "/tmp/.webif/edited-files" -type f 2>&- | wc -l)
	CHANGES=$(($CHANGES + $EDITED_FILES))
}

#######################################################
# beginpage
#######################################################
beginpage() {
	cat <<EOF
Content-Type: text/html; charset=UTF-8
Pragma: no-cache
EOF
	! empty "$SETCOOKIES" && echo "$SETCOOKIES"
	cat <<EOF

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
  <title>PhidgetSBC Configuration</title>
  <link rel="stylesheet" type="text/css" href="/css/webif.css" />
  <!--[if lt IE 7.]>
    <script defer type="text/javascript" src="/js/pngfix.js"></script>
  <![endif]-->
  <meta http-equiv="content-type" content="text/html;charset=utf-8" />
  <meta http-equiv="expires" content="-1" />
$header_inject_head
</head>
<body>
$header_inject_body
EOF
}

#######################################################
# begincontent
#######################################################
begincontent() {
	cat <<EOF
  <div id="content">
    <table width="100%" border="0" cellspacing="0" cellpadding="2">
      <tr valign="top">
        <td>
          <table width="100%" border="0" cellpadding="10" cellspacing="0">
            <tr>
              <td>

EOF
}

#######################################################
# footer
#######################################################
footer() {
	cat <<EOF

<!-- END PAGE SPECIFIC SECTION -->

$_savebutton
EOF
	equal "$didheader" "1" && {
		cat <<EOF

                    </td>
                  </tr>
                </table>
EOF
	}
	cat <<EOF
              </td>
            </tr>
          </table>
        </td>
      </tr>
    </table>
  </div> <!-- End #content -->
$_endform
  <div id="pagefooter">
    <span> &copy; 2021 by <a title="www.phidgets.com" href="http://www.phidgets.com">Phidgets Inc.</a> All rights reserved.</span>
  </div>
</body>
</html>
EOF
}

#######################################################
# messagebox
#######################################################
messagebox() {
	# These can live alone
	! empty "$ERROR" || ! empty "$MESSAGE" || ! empty "$AUTH_MESSAGE" && {
		cat <<EOF
                <div id="infobox">
                  <table border="0" cellspacing="0" cellpadding="4" width="100%">
                    <tr>
                      <td class="icon" align="center" valign="top"><img src="../images/exclam.gif" alt="" /></td>
                      <td class="message">
EOF

		! empty "$ERROR" && {
			echo "                        <p><span style=\"color:#E7292C\">Error(s)</span></p>"
			echo "                        <p class="failed-validation">$ERROR</p><br />"
		}
		! empty "$MESSAGE" && {
			echo "                        <p>$MESSAGE</p>"
		}
		! empty "$AUTH_MESSAGE" && {
			echo "                        <p>$AUTH_MESSAGE</p>"
		}
		
		cat <<EOF
                      </td>
                    </tr>
                  </table>
                </div>
EOF
	}
}

#######################################################
# display_page
#	displays a locally generated page rather then the
#	webif page
#######################################################
display_page()
{
	beginpage
	begincontent

	case "$1" in
	  login)
		cat <<EOF
                <p class="pgtitle" align="center">PhidgetSBC Administration Console</p>
EOF
		messagebox
		cat <<EOF
                <table width="100%" cellspacing="0" cellpadding="0" border="0">
                  <tbody>
                    <tr>
                      <td align="center">
                        <form name="iform" id="iform" action="status-system.sh" method="post">
                          <table>
                            <tr>
                              <td>Username</td>
                              <td><input class="formfld" type="text" name="username" value="" /></td>
                            </tr>
                            <tr>
                              <td>Password</td>
                              <td><input class="formfld" type="password" name="password" value="" /></td>
                            </tr>
                            <tr>
                              <td align="center" colspan="2">
                              <input class="formbtn" type="submit" name="main_login" value="Login" /></td>
                            </tr>
                          </table>
                        </form>
                      </td>
                    </tr>
                  </tbody>
                </table>
EOF
		;;
	  set_root_pass)
		cat <<EOF
                <p class="pgtitle">Set System Password</p>
EOF
		MESSAGE="You haven't set a password for the Web interface.<br />Please enter one now (the user name in your browser will be 'admin')."
		messagebox
		cat <<EOF
                <form enctype="multipart/form-data" action="$SCRIPT_NAME" method="post">
                  <table>
                    <tr>
                      <td>New Password:</td>
                      <td><input type="password" name="passwd1" /></td>
                    </tr>
                    <tr>
                      <td>Confirm Password:</td>
                      <td><input type="password" name="passwd2" /></td>
                    </tr>
                    <tr>
                      <td colspan="2"><br /><input type="submit" name="set_root_pass" value="Set" /></td>
                    </tr>
                  </table>
                </form>
EOF
		;;
	  *)
		messagebox
	esac

	footer
	exit
}

#######################################################
# newsession
#######################################################
newsession() {
	sessionid=`dbus-uuidgen`
	echo "$sessionid" >> /tmp/thttpd_sessions
	SETCOOKIES="Set-Cookie: session=$sessionid"
}

#######################################################
# authenticate
#	Authenticates the user. If they are logged in, it 
#	just passes. Otherwise, ask for a login and
#	authenticate, set up the session, etc.
#
#	Also checks for a blank root password and asks for
#	it to be set.
#
#	return on pass, generate a page and call exit on
#	fail to stop the rest of the page script from
#	running.
#######################################################
authenticate() {

	# XXX - disables authentication for development
	#return

	#Check for blank root password
	curpass=`cat /etc/shadow | grep root | cut -d: -f2`
	empty "$curpass" && {
		! empty "$FORM_set_root_pass" && {
			equal "$FORM_passwd1" "$FORM_passwd2" && {
				validate <<EOF
password|FORM_passwd1|Password|required min=1 max=63|$FORM_passwd1
EOF
				empty "$ERROR" && {
					(
						echo "$FORM_passwd1"
						sleep 1
						echo "$FORM_passwd2"
					) | passwd root 2>&1
					#we have successfully set the password
					AUTH_MESSAGE="Password successfully set."
					newsession
					return
				} || {
					#password didn't validate (probably null)
					display_page set_root_pass
				}
			} || {
				ERROR="The entered passwords do not match!<br />"
				display_page set_root_pass
			}
		}

		display_page set_root_pass
	}
	
	#See if we have a valid session
	sessionid="${HTTP_COOKIE#session=}"
	sessionid="${sessionid%%;*}"
	! empty "$sessionid" && {
		[ -e /tmp/thttpd_sessions ] && grep "$sessionid" /tmp/thttpd_sessions >/dev/null 2>&1 && return
	}
	
	#See if we've just logged in
	if ! empty "$FORM_main_login"; then
		#new login - check credentials
		if [ "$FORM_username" = "admin" ] || [ "$FORM_username" = "Admin" ]; then
			DEBIAN_VERSION=`cat /etc/debian_version`
			DEBIAN_MAJOR_VERSION=${DEBIAN_VERSION%%.*}
			if [ ${DEBIAN_MAJOR_VERSION} -ge 11 ] ; then
				#username must be 'admin', now check password
				salt=`cat /etc/shadow | grep root | cut -d: -f2 | cut -d$ -f 1-4`
				contrl=`mkpasswd $FORM_password $salt`
			else
				#username must be 'admin', now check password
				salt=`cat /etc/shadow | grep root | cut -d: -f2 | cut -d$ -f 3`
				contrl=`mkpasswd -m sha-512 $FORM_password $salt`
			fi
			if [ "$contrl" = "$curpass" ]; then
				#pass auth, create a new session and set a cookie
				newsession
				return
			else
				#bad password
				ERROR='You have specified a bad username or password.<br />'
			fi
		else
			#bad username
			ERROR='You have specified a bad username or password.<br />'
		fi
	fi

	display_page login
}

#######################################################
# apply_saves
#######################################################
apply_saves() {
	update_changes
	_changes=${CHANGES#0}
	_changes=${_changes:+(${_changes})}
	
	#apply any saved settings
	if ! equal "$_changes" ""; then
		. $libdir/apply.sh
		empty "$ERROR" && {
			MESSAGE="The changes have been applied successfully."
		}
	else
		MESSAGE="No changes were made"
	fi
}

#######################################################
# header
#	Displays the page header and menus. Also sets up a
#	FORM surround if needed.
# Arguments:
#	1: Category
#	2: Subcateory
#	3: Page title
#	4: Form action (generally $SCRIPT_NAME)
#######################################################
header() {
	_category="$1"
	_uptime="$(uptime)"
	_loadavg="${_uptime#*load average: }"
	_uptime="${_uptime#*up }"
	_uptime="${_uptime%%,*}"
	_hostname=$(cat /proc/sys/kernel/hostname)
	_version=$( grep "version" /etc/phidgetsbc_version | awk '{print $2}' )
	_board_name="$([ -e /proc/device-tree/model ] && (cat /proc/device-tree/model; echo '';) || (cat /proc/cpuinfo | awk '/^Hardware/ {print $3}'))"
	equal "$_board_name" "PhidgetSBC4" && {
		if [ -e /sys/class/i2c-adapter/i2c-2/2-0050/eeprom ] ; then
			_board_version="$(xxd -s 6 -l 2 -p /sys/class/i2c-adapter/i2c-2/2-0050/eeprom | awk '{print strtonum("0x"$1)}')"
		else
			_board_version="$(xxd -s 6 -l 2 -p /sys/class/i2c-adapter/i2c-1/1-0050/eeprom | awk '{print strtonum("0x"$1)}')"
		fi
	} || {
		_board_version="$(cat /proc/cpuinfo | awk '/^Revision/ {print strtonum("0x"$3)}')"
	}
	_head="${3:+                <p class=\"pgtitle\">$3</p>}"
	_form="${4:+  <form enctype=\"multipart/form-data\" action=\"$4\" method=\"post\"><div id=\"hiddensubmit\"><input type=\"hidden\" name=\"submit\" value=\"1\" /></div>}"
	_categories=$(categories $1)
	_subcategories=${2:+$(subcategories "$1" "$2")}
	_pagename="${2:+$2 - }"
	_savebutton="${4:+                <br /><div class=\"page-save\"><input type=\"submit\" name=\"action\" value=\"Save Changes\"/></div>}"
	_space="${_savebutton:+                <br />}"
	_endform="${_savebutton:+  </form>}"
	equal "$_board_name" "PhidgetSBC4" && {
		_picture=""
	} || {
		_picture="<img src=\"/images/phidgets/1072_0_Thumbnail.png\" alt=\"1072\" />"
	}
	beginpage
	cat <<EOF
  <div id="header">
    <br />
    <h1>PhidgetSBC Configuration</h1>
    <div id="header-picture">
      $_picture
    </div>
    <div id="short-status">
      <ul>
        <li><strong>Host Name:</strong> $_hostname</li>
        <li><strong>Uptime:</strong> $_uptime</li>
        <li><strong>Load:</strong> $_loadavg</li>
        <li><strong>Version:</strong> $_board_version, $_version</li>
      </ul>
    </div>
  </div>

  <div id="mainmenu">
$_categories
  </div>

  <div id="submenu">
$_subcategories
  </div>
  
EOF
	! empty "$_form" && echo "$_form"
	begincontent
	! empty "$_head" && echo "$_head"
	! empty "$_subtitle" && echo "$_subtitle"
	messagebox
	
	cat <<EOF
                <table width="100%" border="0" cellpadding="0" cellspacing="0">
                  <tr>
                    <td class="tabcont">
					  
<!-- BEGIN PAGE SPECIFIC SECTION -->

EOF
	didheader=1
}
# end of header

#######################################################
# display_form
#######################################################
display_form() {
	if empty "$1"; then
		awk -F'|' -f $libdir/common.awk -f $libdir/form.awk
	else
		echo "$1" | awk -F'|' -f $libdir/common.awk -f $libdir/form.awk
	fi
}

#######################################################
# list_remove
#######################################################
list_remove() {
	echo "$1 " | awk '
BEGIN {
	RS=" "
	FS=":"
}
($0 !~ /^'"$2"'/) && ($0 != "") {
	printf " " $0
	first = 0
}'
}

#######################################################
# handle_list
#######################################################
handle_list() {
	# $1 - remove
	# $2 - add
	# $3 - submit
	# $4 - validate
	
	empty "$1" || {
		LISTVAL="$(list_remove "$LISTVAL" "$1") "
		LISTVAL="${LISTVAL# }"
		LISTVAL="${LISTVAL%% }"
		_changed=1
	}
	
	empty "$3" || {
		validate "${4:-none}|$2" && {
			if empty "$LISTVAL"; then
				LISTVAL="$2"
			else
				LISTVAL="$LISTVAL $2"
			fi
			_changed=1
		}
	}

	LISTVAL="${LISTVAL# }"
	LISTVAL="${LISTVAL%% }"
	LISTVAL="${LISTVAL:- }"

	if empty "$_changed"; then
		return 255
	else
		return 0
	fi
}

# First thing to run on the webif pages
authenticate
